@@ -4,36 +4,34 @@
 #' \code{.f} for its side-effect and returns the original
 #' input. \code{map()} returns a list or a data frame; \code{map_lgl()},
 #' \code{map_int()}, \code{map_dbl()} and \code{map_chr()} return vectors
-#' of the corresponding type (or die trying).
+#' of the corresponding type (or die trying); \code{map_df()} returns
+#' a data frame by row-binding the individual elements.
 #'
-#' @param .x A list or vector.
-#' @param .f A function, formula or string.
+#' Note that \code{map()} understands data frames, including grouped
+#' data frames. It can be much faster than
+#' \code{\link[dplyr:summarise_each]{mutate_each()}} when your data frame has many
+#' columns. However, \code{map()}ll be slower for the more common case of many
+#' groups with functions that dplyr knows how to translate to C++.
 #'
-#'   If a function, it is used as is.
-#'
-#'   If a formula, e.g. \code{~ .x + 2}, it is converted to a function with
-#'   a three arguments, \code{.x} or \code{.}, \code{.y}, \code{.z}. This allows
-#'   you to create very compact anonymous functions of up to 3 variables.
-#'
-#'   If a string, e.g. \code{"y"}, it is converted to an extractor function,
-#'   \code{function(x) x[["y"]]}.
+#' @inheritParams as_function
+#' @param .x A list or atomic vector.
 #' @param ... Additional arguments passed on to \code{.f}.
-#' @return \code{map()} a list if \code{.x} is a list or a data frame
-#'   if \code{.x} is a data frame.
+#' @return \code{map()} always returns a list.
 #'
 #'   \code{map_lgl()} returns a logical vector, \code{map_int()} an integer
-#'   vector, \code{map_dbl()}, a numeric vector, \code{map_chr()}, a character
-#'   vector.
+#'   vector, \code{map_dbl()}, a double vector, \code{map_chr()}, a character
+#'   vector. The output of \code{.f} will be automatically typed upwards,
+#'   e.g. logical -> integer -> double -> character.
 #'
 #'   \code{walk()} (invisibly) the input \code{.x}. It's called primarily for
 #'   its side effects, but this makes it easier to combine in a pipe.
-#' @seealso \code{\link{map2}()} and \code{\link{map3}()} to map over multiple
+#' @seealso \code{\link{map2}()} and \code{\link{pmap}()} to map over multiple
 #'   inputs simulatenously
 #' @export
 #' @examples
 #' 1:10 %>%
 #'   map(rnorm, n = 10) %>%
-#'   map(mean, .type = numeric(1))
+#'   map_dbl(mean)
 #'
 #' # Or use an anonymous function
 #' 1:10 %>%
@@ -56,22 +54,35 @@
 #' mtcars %>% map(sum)
 #' # * vector
 #' mtcars %>% map_dbl(sum)
+#'
+#' # If each element of the output is a data frame, use
+#' # map_df to row-bind them together:
+#' mtcars %>%
+#'   split(.$cyl) %>%
+#'   map(~ lm(mpg ~ wt, data = .x)) %>%
+#'   map_df(~ as.data.frame(t(as.matrix(coef(.)))))
+#' # (if you also want to preserve the variable names see
+#' # the broom package)
+#' @useDynLib purrr map_impl map_by_slice_impl
 map <- function(.x, .f, ...) {
   .f <- as_function(.f)
-  lapply(.x, .f, ...) %>% output_hook(.x)
+  .Call(map_impl, environment(), ".x", ".f", "list")
 }
 
 #' @rdname map
-#' @param .p A predicate: either a function, in any of the ways possible
-#'   for \code{.f}, or a logical vector the same length as \code{.x}.
 #' @export
-map_lgl <- function(.x, .p, ...) {
-  if (is.logical(.p)) {
+map_lgl <- function(.x, .f, ...) {
+  .f <- as_function(.f)
+  .Call(map_impl, environment(), ".x", ".f", "logical")
+}
+
+# Internal version of map_lgl() that works with logical vectors
+probe <- function(.x, .p, ...) {
+  if (is_logical(.p)) {
     stopifnot(length(.p) == length(.x))
     .p
   } else {
-    .p <- as_function(.p)
-    vapply(.x, .p, ..., FUN.VALUE = logical(1))
+    map_lgl(.x, .p, ...)
   }
 }
 
@@ -79,23 +90,34 @@ map_lgl <- function(.x, .p, ...) {
 #' @export
 map_chr <- function(.x, .f, ...) {
   .f <- as_function(.f)
-  vapply(.x, .f, ..., FUN.VALUE = character(1))
+  .Call(map_impl, environment(), ".x", ".f", "character")
 }
 
 #' @rdname map
 #' @export
 map_int <- function(.x, .f, ...) {
   .f <- as_function(.f)
-  vapply(.x, .f, ..., FUN.VALUE = integer(1))
+  .Call(map_impl, environment(), ".x", ".f", "integer")
 }
 
 #' @rdname map
 #' @export
 map_dbl <- function(.x, .f, ...) {
   .f <- as_function(.f)
-  vapply(.x, .f, ..., FUN.VALUE = double(1))
+  .Call(map_impl, environment(), ".x", ".f", "double")
 }
 
+#' @rdname map
+#' @param .id If not \code{NULL} a variable with this name will be created
+#'   giving either the name or the index of the data frame.
+#' @export
+map_df <- function(.x, .f, ..., .id = NULL) {
+  .f <- as_function(.f)
+  res <- map(.x, .f, ...)
+  dplyr::bind_rows(res, .id = .id)
+}
+
+
 #' @export
 #' @rdname map
 walk <- function(.x, .f, ...) {
@@ -109,22 +131,26 @@ walk <- function(.x, .f, ...) {
 
 #' Map over multiple inputs simultaneously.
 #'
-#' These functions are designed in such a way that arguments to be
-#' vectorised over come before the function name, and arguments that
-#' should be supplied to every call come after the function name.
+#' These functions are variants of \code{map()} iterate over multiple
+#' arguments in parallel. \code{map2} is specialised for the two argument
+#' case; \code{pmap} allows you to provide any number of arguments in a
+#' list.
 #'
-#' \code{map_n()} and \code{walk_n()} take a single list \code{.l} and
-#' map over all its elements simultaneously. \code{map2()} and
-#' \code{map3()} return a data frame when \code{.x} is a data frame.
+#' Note that arguments to be vectorised over come before the \code{.f},
+#' and arguments that are supplied to every call come after \code{.f}.
+#'
+#' \code{pmap()} and \code{pwalk()} take a single list \code{.l} and
+#' map over all its elements in parallel.
 #'
 #' @inheritParams map
-#' @param .f A function of two (for \code{map2} and \code{walk2}) or
-#' three (\code{map3} and \code{walk3}) arguments. For \code{map_n}
-#' and \code{walk_n}, the number of arguments must correspond to the
-#' number of elements of \code{.l}.
-#' @param .x,.y,.z Lists of the same length or of length 1. Only
-#' lists of length 1 are recycled.
-#' @param .l A list of lists to be mapped on simultaneously.
+#' @param .x,.y Vectors of the same length. A vector of length 1 will
+#'   be recycled.
+#' @param .l A list of lists. The length of \code{.l} determines the
+#'   number of arguments that \code{.f} will be called with. List
+#'   names will be used if present.
+#' @return An atomic vector, list, or data frame, depending on the suffix.
+#'   Atomic vectors and lists will be named if \code{.x} or the first
+#'   element of \code{.l} is named.
 #' @export
 #' @examples
 #' x <- list(1, 10, 100)
@@ -133,59 +159,144 @@ walk <- function(.x, .f, ...) {
 #' # Or just
 #' map2(x, y, `+`)
 #'
-#' z <- list(15, 20, 25)
-#' map3(x, y, z, ~ .x ^ .y + .z)
-#'
 #' # Split into pieces, fit model to each piece, then predict
 #' by_cyl <- mtcars %>% split(.$cyl)
 #' mods <- by_cyl %>% map(~ lm(mpg ~ wt, data = .))
 #' map2(mods, by_cyl, predict)
+#' @useDynLib purrr map2_impl
 map2 <- function(.x, .y, .f, ...) {
-  map_n(list(.x, .y), .f, ...) %>% output_hook(.x)
-}
-
-#' @export
-#' @rdname map2
-map3 <- function(.x, .y, .z, .f, ...) {
-  map_n(list(.x, .y, .z), .f, ...) %>% output_hook(.x)
-}
-
-#' @export
-#' @rdname map2
-map_n <- function(.l, .f, ...) {
   .f <- as_function(.f)
-  args <- recycle_args(.l)
-  do.call("mapply", c(
-    FUN = list(quote(.f)), args, MoreArgs = quote(list(...)),
-    SIMPLIFY = FALSE, USE.NAMES = FALSE
-  ))
+  .Call(map2_impl, environment(), ".x", ".y", ".f", "list")
 }
 
+#' @export
+#' @rdname map2
+map2_lgl <- function(.x, .y, .f, ...) {
+  .f <- as_function(.f)
+  .Call(map2_impl, environment(), ".x", ".y", ".f", "logical")
+}
+#' @export
+#' @rdname map2
+map2_int <- function(.x, .y, .f, ...) {
+  .f <- as_function(.f)
+  .Call(map2_impl, environment(), ".x", ".y", ".f", "integer")
+}
+#' @export
+#' @rdname map2
+map2_dbl <- function(.x, .y, .f, ...) {
+  .f <- as_function(.f)
+  .Call(map2_impl, environment(), ".x", ".y", ".f", "double")
+}
+#' @export
+#' @rdname map2
+map2_chr <- function(.x, .y, .f, ...) {
+  .f <- as_function(.f)
+  .Call(map2_impl, environment(), ".x", ".y", ".f", "character")
+}
+#' @rdname map2
+#' @export
+map2_df <- function(.x, .y, .f, ..., .id = NULL) {
+  .f <- as_function(.f)
+  res <- map2(.x, .y, .f, ...)
+  dplyr::bind_rows(res, .id = .id)
+}
+
+#' @export
+#' @rdname map2
+#' @usage NULL
+map3 <- function(.x, .y, .z, .f, ...) {
+  warning("`map3(x, y, z)` is deprecated. Please use `pmap(list(x, y, z))` ",
+    "instead", call. = FALSE)
+  pmap(list(.x, .y, .z), .f, ...)
+}
+
+#' @export
+#' @rdname map2
+#' @useDynLib purrr pmap_impl
+pmap <- function(.l, .f, ...) {
+  .f <- as_function(.f)
+  .Call(pmap_impl, environment(), ".l", ".f", "list")
+}
+
+#' @export
+#' @rdname map2
+pmap_lgl <- function(.l, .f, ...) {
+  .f <- as_function(.f)
+  .Call(pmap_impl, environment(), ".l", ".f", "logical")
+}
+#' @export
+#' @rdname map2
+pmap_int <- function(.l, .f, ...) {
+  .f <- as_function(.f)
+  .Call(pmap_impl, environment(), ".l", ".f", "integer")
+}
+#' @export
+#' @rdname map2
+pmap_dbl <- function(.l, .f, ...) {
+  .f <- as_function(.f)
+  .Call(pmap_impl, environment(), ".l", ".f", "double")
+}
+#' @export
+#' @rdname map2
+pmap_chr <- function(.l, .f, ...) {
+  .f <- as_function(.f)
+  .Call(pmap_impl, environment(), ".l", ".f", "character")
+}
+#' @rdname map2
+#' @export
+pmap_df <- function(.l, .f, ..., .id = NULL) {
+  .f <- as_function(.f)
+  res <- map(.l, .f, ...)
+  dplyr::bind_rows(res, .id = .id)
+}
+
+
+#' @export
+#' @usage NULL
+#' @rdname map2
+map_n <- function(...) {
+  warning("`map_n()` is deprecated; please use `pmap()` instead.",
+    call. = FALSE)
+  pmap(...)
+}
 
 #' @export
 #' @rdname map2
 walk2 <- function(.x, .y, .f, ...) {
-  walk_n(list(.x, .y), .f, ...)
+  pwalk(list(.x, .y), .f, ...)
   invisible(.x)
 }
 
 #' @export
+#' @usage NULL
 #' @rdname map2
 walk3 <- function(.x, .y, .z, .f, ...) {
-  walk_n(list(.x, .y, .z), .f, ...)
+  warning("`walk3(x, y, z)` is deprecated. Please use `pwalk(list(x, y, z))` ",
+    "instead", call. = FALSE)
+  pwalk(list(.x, .y, .z), .f, ...)
   invisible(.x)
 }
 
 #' @export
 #' @rdname map2
-walk_n <- function(.l, .f, ...) {
-  args_list <- recycle_args(.l) %>% zip_n()
+pwalk <- function(.l, .f, ...) {
+  .f <- as_function(.f)
+  args_list <- recycle_args(.l) %>% transpose()
   for (args in args_list) {
     do.call(".f", c(args, list(...)))
   }
   invisible(.l)
 }
 
+#' @export
+#' @rdname map2
+#' @usage NULL
+walk_n <- function(...) {
+  warning("`walk_n()` is deprecated; please use `pwalk()` instead.",
+    call. = FALSE)
+  pwalk(...)
+}
+
 #' Modify elements conditionally
 #'
 #' \code{map_if()} maps a function over the elements of \code{.x}
@@ -202,14 +313,9 @@ walk_n <- function(.l, .f, ...) {
 #' @param .at A character vector of names or a numeric vector of
 #'   positions. Only those elements corresponding to \code{.at} will be
 #'   modified.
-#' @return The same type of object as \code{.x}.
+#' @return A list.
 #' @name conditional-map
 #' @examples
-#' list(x = rbernoulli(100), y = 1:100) %>%
-#'   zip_n() %>%
-#'   map_if("x", ~ update_list(., y = ~ y * 100)) %>%
-#'   zip_n(.simplify = TRUE)
-#'
 #' # Convert factors to characters
 #' iris %>%
 #'   map_if(is.factor, as.character) %>%
@@ -220,12 +326,20 @@ walk_n <- function(.l, .f, ...) {
 #'
 #' # Or with a vector of names:
 #' mtcars %>% map_at(c("cyl", "am"), as.character) %>% str()
+#'
+#' list(x = rbernoulli(100), y = 1:100) %>%
+#'   transpose() %>%
+#'   map_if("x", ~ update_list(., y = ~ y * 100)) %>%
+#'   transpose() %>%
+#'   simplify_all()
+#'
 NULL
 
 #' @rdname conditional-map
 #' @export
 map_if <- function(.x, .p, .f, ...) {
-  sel <- map_lgl(.x, .p)
+  .x <- c(.x)
+  sel <- probe(.x, .p)
   .x[sel] <- map(.x[sel], .f, ...)
   .x
 }
@@ -233,6 +347,7 @@ map_if <- function(.x, .p, .f, ...) {
 #' @rdname conditional-map
 #' @export
 map_at <- function(.x, .at, .f, ...) {
+  .x <- c(.x)
   sel <- inv_which(.x, .at)
   .x[sel] <- map(.x[sel], .f, ...)
   .x
@@ -252,24 +367,3 @@ inv_which <- function(x, sel) {
   }
 }
 
-#' Map a list to a function call
-#'
-#' While \code{\link{lift_dl}()} wraps a function in
-#' \code{\link{do.call}()}, \code{map_call()} is directly equivalent
-#' to \code{do.call()} except that it takes a list as first argument
-#' instead of a function. This makes `map_call()` pipable.
-#' @param .x A list or a vector. Vectors are coerced to a list.
-#' @param .f A function or the name of a function to call with the
-#'   elements of \code{.x} as arguments.
-#' @param ... Additional arguments passed on to \code{.f}.
-#' @seealso \code{\link{lift_dl}()} and \code{\link{do.call}()}
-#' @export
-#' @examples
-#' # We map a list of strings to paste(), with sep = "-" and the
-#' # string "2001" as additional arguments
-#' list("01", "01") %>%
-#'   map(~ sub("^01", "10", .)) %>%
-#'   map_call("paste", "2001", sep = "-")
-map_call <- function(.x, .f, ...) {
-  do.call(.f, c(.x, list(...)))
-}
